@@ -1,7 +1,7 @@
 #!/usr/bin/env python3
 #
 # Copyright © 2008 Henri Hakkinen
-# Copyright © 2015-2017 Arun Prakash Jana <engineerarun@gmail.com>
+# Copyright © 2015-2018 Arun Prakash Jana <engineerarun@gmail.com>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -30,10 +30,11 @@ from http.client import HTTPSConnection
 import locale
 import logging
 import os
+import shutil
 import signal
 import socket
 import ssl
-import subprocess
+from subprocess import Popen, PIPE, DEVNULL
 import sys
 import textwrap
 import urllib.parse
@@ -67,7 +68,7 @@ signal.signal(signal.SIGINT, sigint_handler)
 
 # Constants
 
-_VERSION_ = '3.4'
+_VERSION_ = '3.5'
 
 COLORMAP = {k: '\x1b[%sm' % v for k, v in {
     'a': '30', 'b': '31', 'c': '32', 'd': '33',
@@ -82,7 +83,7 @@ COLORMAP = {k: '\x1b[%sm' % v for k, v in {
 }.items()}
 
 # Disguise as Firefox on Ubuntu
-USER_AGENT = ('Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:56.0) Gecko/20100101 Firefox/56.0')
+USER_AGENT = ('Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:58.0) Gecko/20100101 Firefox/58.0')
 ua = True  # User Agent is enabled by default
 
 text_browsers = ['elinks', 'links', 'lynx', 'w3m', 'www-browser']
@@ -117,7 +118,7 @@ def open_url(url):
     elinks, links, lynx, w3m or 'www-browser'; or True otherwise.
 
     The string attribute ``open_url.override_text_browser`` can be used to
-    ignore env var BROWSER as well as some known text based browsers and
+    ignore env var BROWSER as well as some known text-based browsers and
     attempt to open url in a GUI browser available.
     Note: If a GUI browser is indeed found, this option ignores the program
           option `show-browser-logs`
@@ -126,7 +127,7 @@ def open_url(url):
 
     # Custom URL handler gets max priority
     if hasattr(open_url, 'url_handler'):
-        p = subprocess.Popen([open_url.url_handler, url], stdin=subprocess.PIPE)
+        p = Popen([open_url.url_handler, url], stdin=PIPE)
         p.communicate()
         return
 
@@ -529,7 +530,7 @@ class GoogleUrl(object):
             raise ValueError('Already at the first page.')
         self._start = 0
 
-    # Data source: https://en.wikipedia.org/wiki/List_of_Google_domains
+    # Data source: https://web.archive.org/web/20170615200243/https://en.wikipedia.org/wiki/List_of_Google_domains
     # Scraper script: https://gist.github.com/zmwangx/b976e83c14552fe18b71
     TLD_TO_DOMAIN_MAP = {
         'ac': 'google.ac',      'ad': 'google.ad',      'ae': 'google.ae',
@@ -1592,6 +1593,7 @@ class Result(object):
 
     # Class variables
     colors = None
+    urlexpand = True
 
     def __init__(self, index, title, url, abstract, metadata=None, sitelinks=None):
         index = str(index)
@@ -1613,15 +1615,25 @@ class Result(object):
     def _print_title_and_url(self, index, title, url, indent=0):
         colors = self.colors
 
+        if not self.urlexpand:
+            segments = urllib.parse.urlparse(url)
+            url = '  [' + segments.netloc + ']'
+
         # Pad index and url with `indent` number of spaces
         index = ' ' * indent + str(index)
         url = ' ' * indent + url
         if colors:
             print(colors.index + index + colors.reset, end='')
-            print(' ' + colors.title + title + colors.reset)
-            print(colors.url + url + colors.reset)
+            if self.urlexpand:
+                print(' ' + colors.title + title + colors.reset)
+                print(colors.url + url + colors.reset)
+            else:
+                print(' ' + colors.title + title + colors.reset + colors.url + url + colors.reset)
         else:
-            print(' %s %s\n%s' % (index, title, url))
+            if self.urlexpand:
+                print(' %s %s\n%s' % (index, title, url))
+            else:
+                print(' %s %s%s' % (index, title, url))
 
     def _print_metadata_and_abstract(self, abstract, metadata=None, indent=0):
         colors = self.colors
@@ -1770,6 +1782,8 @@ class GooglerCmd(object):
         self._results_filtered = False
         self._urltable = {}
 
+        self.promptcolor = True if os.getenv('DISABLE_PROMPT_COLOR') is None else False
+
     @property
     def options(self):
         """Current options."""
@@ -1905,12 +1919,11 @@ class GooglerCmd(object):
         """
         colors = self.colors
         message = 'googler (? for help)'
-        prompt = (colors.prompt + message + colors.reset + ' ') if colors else (message + ': ')
+        prompt = (colors.prompt + message + colors.reset + ' ') if (colors and self.promptcolor) else (message + ': ')
         enter_count = 0
         while True:
             try:
-                sys.stderr.write(prompt)
-                cmd = input()
+                cmd = input(prompt)
             except EOFError:
                 sys.exit(0)
 
@@ -1935,7 +1948,12 @@ class GooglerCmd(object):
     @require_keywords
     @no_argument
     def do_first(self):
-        self._google_url.first_page()
+        try:
+            self._google_url.first_page()
+        except ValueError as e:
+            print(e, file=sys.stderr)
+            return
+
         self.fetch_and_display()
 
     def do_google(self, arg):
@@ -2058,6 +2076,32 @@ class GooglerCmd(object):
                     open_url(self._urltable[cmd])
                 elif self.keywords and cmd.isdigit() and int(cmd) < 100:
                     printerr('Index out of bound. To search for the number, use g.')
+                elif cmd == 'u':
+                    Result.urlexpand = not Result.urlexpand
+                    printerr('url expansion toggled.')
+                elif cmd.startswith('c ') and cmd[2:].isdigit():
+                    try:
+                        # try copying the url to clipboard using native utilities
+                        if sys.platform.startswith(('linux', 'freebsd', 'openbsd')):
+                            if shutil.which('xsel') is None:
+                                raise FileNotFoundError
+                            copier_params = ['xsel', '-b', '-i']
+                        elif sys.platform == 'darwin':
+                            copier_params = ['pbcopy']
+                        elif sys.platform == 'win32':
+                            copier_params = ['clip']
+                        else:
+                            copier_params = []
+
+                        if not copier_params:
+                            printerr('operating system not identified')
+                        else:
+                            Popen(copier_params, stdin=PIPE, stdout=DEVNULL,
+                                  stderr=DEVNULL).communicate(self._urltable[cmd[2:]].encode('utf-8'))
+                    except FileNotFoundError:
+                        printerr('xsel missing')
+                    except Exception:
+                        raise NoKeywordsException
                 else:
                     self.do_google(cmd)
             except NoKeywordsException:
@@ -2082,6 +2126,8 @@ class GooglerArgumentParser(argparse.ArgumentParser):
           O [index|range|a ...] like key 'o', but try to open in a GUI browser
           g keywords            new Google search for 'keywords' with original options
                                 should be used to search omniprompt keys and indices
+          c index               copy url to clipboard
+          u                     toggle url expansion
           q, ^D, double Enter   exit googler
           ?                     show omniprompt help
           *                     other inputs issue a new search with original options
@@ -2094,7 +2140,7 @@ class GooglerArgumentParser(argparse.ArgumentParser):
         file.write(textwrap.dedent("""
         Version %s
         Copyright © 2008 Henri Hakkinen
-        Copyright © 2015-2017 Arun Prakash Jana <engineerarun@gmail.com>
+        Copyright © 2015-2018 Arun Prakash Jana <engineerarun@gmail.com>
         Zhiming Wang <zmwangx@gmail.com>
         License: GPLv3
         Webpage: https://github.com/jarun/googler
@@ -2398,7 +2444,7 @@ def parse_args(args=None, namespace=None):
            help='show results from news section')
     addarg('-c', '--tld', metavar='TLD',
            help="""country-specific search with top-level domain .TLD, e.g., 'in'
-           for India. Ref: https://en.wikipedia.org/wiki/List_of_Google_domains""")
+           for India""")
     addarg('-l', '--lang', metavar='LANG', help='display in language LANG')
     addarg('-x', '--exact', action='store_true',
            help='disable automatic spelling correction')
@@ -2478,6 +2524,7 @@ def main():
         else:
             colors = None
         Result.colors = colors
+        Result.urlexpand = True if os.getenv('DISABLE_URL_EXPANSION') is None else False
         GooglerCmd.colors = colors
 
         if opts.url_handler is not None:
